/*Copyright (C) 2007   Authors: Jacobo Tarragón Cros, Víctor Muñoz Ferrara, William Viana Soares. 
Email addresses: tarragon[Aŧ]aditel[DOT]org, victorm[AT]aditel[DOT]org, wviana[AT]aditel[DOT]org
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

#include "jukebox.h"
#include <iostream>

// Compile command: g++ -Wall -O2 `xine-config --cflags` `xine-config --libs` jukebox.cpp main.cpp -o loquesea

using namespace std;

Jukebox::Jukebox(){

  xine = xine_new();
  xine_init(xine);
  audio_driver = "auto";
  if(!xine)
    cerr << "[ Jukebox-error ] : No se puede inicializar el motor del xine. Comprueba tu instalaci� \n";
  else{
    if(!inicializarStream())
      cerr << "[ Jukebox-error ] : Imposible reproducir el archivo\n";
  }
}

bool Jukebox::inicializarStream(){

  audio_port = xine_open_audio_driver(xine, audio_driver, NULL);
  
  if(!audio_port){
    
    cerr << "[ Jukebox-error ] : No se puede inicializar driver de sonido \n";
    return false;
  }

  stream = xine_stream_new(xine, audio_port, NULL);
  
  if(!stream){
    cerr << "[ Jukebox-error ] : No se puede inicializar stream de sonido\n";
    return false;
  }

  event_queue = xine_event_new_queue(stream);
  
  return true;

}

int Jukebox::duracion()const{
  
  int pos;
  int tiempo_actual;
  int tiempo_total;
  
  xine_get_pos_length(stream, &pos, &tiempo_actual, &tiempo_total);
  return tiempo_total;
}

bool Jukebox::play(char *mrl, int tiempo_inicio){
  
  if(!xine_open(stream,mrl)){
    cerr << "[ Jukebox-Error ] : no se puede abrir '"<<mrl<<"'\n";
    return false;
  }

  if(tiempo_inicio == -1){
    tiempo_inicio = duracion() * 40/100;
//     cout << tiempo_inicio << endl;
  }

  if(!xine_play(stream, 0, tiempo_inicio)){
    cerr << "[ Jukebox-Error ] : no se puede abrir '"<<mrl<<"'\n";
    return false;
  }
  
  //TODO: dar mas informacion sobre los posibles errores usando xine_get_error(stream), aunque no es critico.

  return true;
}

void Jukebox::stop(){
  
  xine_stop(stream);
  xine_set_param(stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
  
}

Jukebox::~Jukebox(){

  if(stream) xine_close(stream);
  if(event_queue) xine_event_dispose_queue(event_queue);
  if(stream) xine_dispose(stream);
  if(audio_port) xine_close_audio_driver(xine, audio_port);
  if(xine) xine_exit(xine);

}

